@@ -1,4 +1,4 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 4 -*- */
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
  * Copyright (C) 2001-2002 Mikael Hallendal <micke@imendio.com>
  * Copyright (C) 2003,2004 Shaun McCance <shaunm@gnome.org>
@@ -31,7 +31,6 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
-#include <glade/glade.h>
 #include <string.h>
 #include <libgnome/gnome-config.h>
 #include <libgnome/gnome-url.h>
@@ -43,6 +42,7 @@
 #include "yelp-toc.h"
 #include "yelp-docbook.h"
 #include "yelp-db-print.h"
+#include "yelp-mallard.h"
 #include "yelp-window.h"
 #include "yelp-print.h"
 #include "yelp-debug.h"
@@ -53,10 +53,12 @@
 #include "yelp-search.h"
 #include "gtkentryaction.h"
 
-#define YELP_CONFIG_WIDTH          "/yelp/Geometry/width"
-#define YELP_CONFIG_HEIGHT         "/yelp/Geometry/height"
-#define YELP_CONFIG_WIDTH_DEFAULT  "600"
-#define YELP_CONFIG_HEIGHT_DEFAULT "420"
+#define YELP_CONFIG_PATH           "/.gnome2/yelp"
+#define YELP_CONFIG_GEOMETRY_GROUP "Geometry"
+#define YELP_CONFIG_WIDTH          "width"
+#define YELP_CONFIG_HEIGHT         "height"
+#define YELP_CONFIG_WIDTH_DEFAULT  600
+#define YELP_CONFIG_HEIGHT_DEFAULT 420
 
 #define BUFFER_SIZE 16384
 
@@ -103,10 +105,10 @@ static gboolean    window_do_load_html            (YelpWindow        *window,
 						   YelpRrnType        type,
 						   gboolean           need_history);
 static void        window_set_loading             (YelpWindow        *window);
-static void        window_setup_window            (YelpWindow        *window, 
+static void        window_setup_window            (YelpWindow        *window,
 						   YelpRrnType        type,
-						   gchar             *loading_uri, 
-						   gchar             *frag, 
+						   gchar             *loading_uri,
+						   gchar             *frag,
 						   gchar             *req_uri,
 						   gchar             *base_uri,
 						   gboolean           add_history);
@@ -214,14 +216,14 @@ static void        window_find_clicked_cb         (GtkWidget         *button,
 static gboolean    window_find_hide_cb            (GtkWidget *widget,
 						   GdkEventFocus *event,
 						   YelpWindow *window);
-static void        window_find_next_cb            (GtkAction *action, 
+static void        window_find_next_cb            (GtkAction *action,
 						   YelpWindow *window);
-static void        window_find_previous_cb        (GtkAction *action, 
+static void        window_find_previous_cb        (GtkAction *action,
 						   YelpWindow *window);
 static gboolean    tree_model_iter_following      (GtkTreeModel      *model,
 						   GtkTreeIter       *iter);
 static gboolean    window_write_html              (YelpLoadData      *data);
-static void        window_write_print_html        (YelpHtml          *html, 
+static void        window_write_print_html        (YelpHtml          *html,
 						   YelpPage          *page);
 
 enum {
@@ -438,9 +440,9 @@ static const GtkActionEntry entries[] = {
       NULL,
       NULL,
       G_CALLBACK (window_copy_link_cb) },
-    { "Contents", GTK_STOCK_HELP,  
-      N_("_Contents"), 
-      "F1", 
+    { "Contents", GTK_STOCK_HELP,
+      N_("_Contents"),
+      "F1",
       N_("Help On this application"),
       G_CALLBACK (window_help_contents_cb) },
     { "About", GTK_STOCK_ABOUT,
@@ -474,7 +476,7 @@ yelp_window_get_type (void)
 	};
 
 	window_type = g_type_register_static (GTK_TYPE_WINDOW,
-					      "YelpWindow", 
+					      "YelpWindow",
 					      &window_info, 0);
     }
 
@@ -488,10 +490,33 @@ window_init (YelpWindow *window)
 
     window->priv = YELP_WINDOW_GET_PRIVATE (window);
 
-    width = gnome_config_get_int (YELP_CONFIG_WIDTH
-				  "=" YELP_CONFIG_WIDTH_DEFAULT);
-    height = gnome_config_get_int (YELP_CONFIG_HEIGHT
-				   "=" YELP_CONFIG_HEIGHT_DEFAULT);
+    GKeyFile *keyfile = g_key_file_new();
+    GError *config_error = NULL;
+    gchar* config_path = g_strconcat( g_get_home_dir(), YELP_CONFIG_PATH, NULL);
+
+    if( !g_key_file_load_from_file (keyfile, config_path,
+			    G_KEY_FILE_KEEP_COMMENTS | G_KEY_FILE_KEEP_TRANSLATIONS,
+			    &config_error)) {
+	g_warning ("Failed to load config file: %s\n", config_error->message);
+	g_error_free (config_error);
+
+	width = YELP_CONFIG_WIDTH_DEFAULT;
+	height = YELP_CONFIG_HEIGHT_DEFAULT;
+    } else {
+	width = g_key_file_get_integer (keyfile, YELP_CONFIG_GEOMETRY_GROUP,
+			YELP_CONFIG_WIDTH, NULL);
+	height = g_key_file_get_integer (keyfile, YELP_CONFIG_GEOMETRY_GROUP,
+			YELP_CONFIG_HEIGHT, NULL);
+
+	if (width == 0)
+	    width = YELP_CONFIG_WIDTH_DEFAULT;
+	if (height == 0)
+	    height = YELP_CONFIG_HEIGHT_DEFAULT;
+    }
+
+    g_free (config_path);
+    g_key_file_free (keyfile);
+
     gtk_window_set_default_size (GTK_WINDOW (window), width, height);
     g_signal_connect (window,
 		      "configure-event",
@@ -499,7 +524,7 @@ window_init (YelpWindow *window)
 		      NULL);
 
     gtk_window_set_title (GTK_WINDOW (window), _("Help Browser"));
-   
+
     window_populate (window);
 }
 
@@ -519,9 +544,13 @@ window_finalize (GObject *object)
     g_object_unref (priv->ui_manager);
 
     g_free (priv->find_string);
-    
+
     g_free (priv->current_frag);
 
+    g_free (priv->uri);
+    g_free (priv->base_uri);
+    g_free (priv->req_uri);
+
     /* FIXME there are many more things to free */
 
     parent_class->finalize (object);
@@ -575,11 +604,11 @@ history_push_back (YelpWindow *window)
 
     entry->frag_id  = g_strdup (priv->current_frag);
     entry->uri = g_strdup (priv->uri);
-    entry->req_uri = priv->req_uri;
+    entry->req_uri = g_strdup(priv->req_uri);
     entry->doc = priv->current_document;
     entry->type = priv->current_type;
     entry->base_uri = g_strdup (priv->base_uri);
-    
+
     /* page_title, frag_title */
 
     priv->history_back = g_slist_prepend (priv->history_back, entry);
@@ -595,17 +624,17 @@ history_push_back (YelpWindow *window)
 	entry->page_title = g_strdup (title);
 
     entry->window = window;
-    
+
     entry->menu_entry = gtk_menu_item_new_with_label (entry->page_title);
     g_object_ref (entry->menu_entry);
 
-    entry->callback = g_signal_connect (G_OBJECT (entry->menu_entry), 
-		      "activate", 
+    entry->callback = g_signal_connect (G_OBJECT (entry->menu_entry),
+		      "activate",
 		      G_CALLBACK (history_back_to),
 		      entry);
-		      
 
-    gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->back_menu), 
+
+    gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->back_menu),
 			    entry->menu_entry);
     gtk_widget_show (entry->menu_entry);
 }
@@ -626,7 +655,7 @@ history_push_forward (YelpWindow *window)
 
     entry->frag_id  = g_strdup (priv->current_frag);
     entry->uri = g_strdup (priv->uri);
-    entry->req_uri = priv->req_uri;
+    entry->req_uri = g_strdup(priv->req_uri);
     entry->doc = priv->current_document;
     entry->type = priv->current_type;
     entry->base_uri = g_strdup (priv->base_uri);
@@ -650,13 +679,13 @@ history_push_forward (YelpWindow *window)
     entry->menu_entry = gtk_menu_item_new_with_label (entry->page_title);
     g_object_ref (entry->menu_entry);
 
-    entry->callback = g_signal_connect (G_OBJECT (entry->menu_entry), 
-		      "activate", 
+    entry->callback = g_signal_connect (G_OBJECT (entry->menu_entry),
+		      "activate",
 		      G_CALLBACK (history_forward_to),
 		      entry);
-		      
 
-    gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->forward_menu), 
+
+    gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->forward_menu),
 			    entry->menu_entry);
     gtk_widget_show (entry->menu_entry);
 }
@@ -696,10 +725,8 @@ history_step_back (YelpWindow *window)
     priv = window->priv;
     entry = history_pop_back (window);
 
-    if (priv->current_frag) {
-	g_free (priv->current_frag);
-	priv->current_frag = NULL;
-    }
+    g_free (priv->current_frag);
+    priv->current_frag = NULL;
 
     if (entry) {
 	priv->current_frag = g_strdup (entry->frag_id);
@@ -768,6 +795,8 @@ history_entry_free (YelpHistoryEntry *entry)
     g_free (entry->page_title);
     g_free (entry->frag_title);
     g_free (entry->base_uri);
+    g_free (entry->uri);
+    g_free (entry->req_uri);
 
     if (entry->menu_entry) {
 	gtk_widget_destroy (entry->menu_entry);
@@ -776,7 +805,7 @@ history_entry_free (YelpHistoryEntry *entry)
     g_free (entry);
 }
 
-static void 
+static void
 history_back_to (GtkMenuItem *menuitem, YelpHistoryEntry *entry)
 {
     YelpWindow       *window;
@@ -792,21 +821,21 @@ history_back_to (GtkMenuItem *menuitem, YelpHistoryEntry *entry)
 
     while (!g_str_equal (latest->page_title, entry->page_title)) {
 	priv->history_forward = g_slist_prepend (priv->history_forward, latest);
-	gtk_container_remove (GTK_CONTAINER (priv->back_menu), 
+	gtk_container_remove (GTK_CONTAINER (priv->back_menu),
 			      latest->menu_entry);
-	
+
 	g_signal_handler_disconnect (latest->menu_entry, latest->callback);
-	latest->callback = g_signal_connect (G_OBJECT (latest->menu_entry), 
-					     "activate", 
+	latest->callback = g_signal_connect (G_OBJECT (latest->menu_entry),
+					     "activate",
 					     G_CALLBACK (history_forward_to),
 					     latest);
-	
-	
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->forward_menu), 
+
+
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->forward_menu),
 				latest->menu_entry);
 	gtk_widget_show (latest->menu_entry);
 
-	
+
 	latest = history_pop_back (window);
     }
     if (latest->menu_entry) {
@@ -815,6 +844,7 @@ history_back_to (GtkMenuItem *menuitem, YelpHistoryEntry *entry)
     }
 
     history_load_entry (window, entry);
+    history_entry_free (entry);
 }
 
 static void
@@ -833,20 +863,20 @@ history_forward_to (GtkMenuItem *menuitem, YelpHistoryEntry *entry)
 
     while (!g_str_equal (latest->page_title, entry->page_title)) {
 	priv->history_back = g_slist_prepend (priv->history_back, latest);
-	gtk_container_remove (GTK_CONTAINER (priv->forward_menu), 
+	gtk_container_remove (GTK_CONTAINER (priv->forward_menu),
 			      latest->menu_entry);
-	
+
 	g_signal_handler_disconnect (latest->menu_entry, latest->callback);
-	latest->callback = g_signal_connect (G_OBJECT (latest->menu_entry), 
-					     "activate", 
+	latest->callback = g_signal_connect (G_OBJECT (latest->menu_entry),
+					     "activate",
 					     G_CALLBACK (history_back_to),
 					     latest);
 
-	gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->back_menu), 
+	gtk_menu_shell_prepend (GTK_MENU_SHELL (window->priv->back_menu),
 				latest->menu_entry);
 	gtk_widget_show (latest->menu_entry);
 
-	
+
 	latest = history_pop_forward (window);
     }
     if (latest->menu_entry) {
@@ -855,6 +885,8 @@ history_forward_to (GtkMenuItem *menuitem, YelpHistoryEntry *entry)
     }
 
     history_load_entry (window, entry);
+    history_entry_free (entry);
+
 
 }
 
@@ -875,7 +907,7 @@ page_request_cb (YelpDocument       *document,
 	       YelpWindow         *window)
 {
     YelpError *error;
-    YelpLoadData *data;
+    YelpLoadData *data = NULL;
 
     switch (signal) {
     case YELP_DOCUMENT_SIGNAL_PAGE:
@@ -890,6 +922,8 @@ page_request_cb (YelpDocument       *document,
 	window->priv->current_request = -1;
 	yelp_page_free ((YelpPage *) func_data);
 	gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
+
+	g_free (data);
 	break;
     case YELP_DOCUMENT_SIGNAL_TITLE:
 	/* We don't need to actually handle title signals as gecko
@@ -899,7 +933,7 @@ page_request_cb (YelpDocument       *document,
 	break;
     case YELP_DOCUMENT_SIGNAL_ERROR:
 	error = (YelpError *) func_data;
-	window_error (window, (gchar *) yelp_error_get_title (error), 
+	window_error (window, (gchar *) yelp_error_get_title (error),
 		      (gchar *) yelp_error_get_message (error), FALSE);
 	yelp_error_free (error);
 	gdk_window_set_cursor (GTK_WIDGET (window)->window, NULL);
@@ -923,7 +957,7 @@ window_setup_window (YelpWindow *window, YelpRrnType type,
      */
     YelpWindowPriv *priv;
     GtkAction      *action;
- 
+
     g_return_if_fail (YELP_IS_WINDOW (window));
 
     priv = window->priv;
@@ -944,29 +978,31 @@ window_setup_window (YelpWindow *window, YelpRrnType type,
     priv->current_type = type;
     g_free (priv->uri);
     priv->uri = g_strdup (loading_uri);
+    g_free (priv->current_frag);
     priv->current_frag = g_strdup (frag);
+    g_free (priv->req_uri);
     priv->req_uri = g_strdup (req_uri);
-    
+
     switch (priv->current_type) {
     case YELP_RRN_TYPE_DOC:
-	action = gtk_action_group_get_action (window->priv->action_group, 
+	action = gtk_action_group_get_action (window->priv->action_group,
 					      "PrintDocument");
 	g_object_set (G_OBJECT (action), "sensitive", TRUE, NULL);
-	
+
 	action  = gtk_action_group_get_action (window->priv->action_group,
 					       "AboutDocument");
 	g_object_set (G_OBJECT (action),  "sensitive", TRUE, NULL);
 	break;
     default:
-	action = gtk_action_group_get_action (window->priv->action_group, 
+	action = gtk_action_group_get_action (window->priv->action_group,
 					      "PrintDocument");
 	g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
-	
+
 	action  = gtk_action_group_get_action (window->priv->action_group,
 					       "AboutDocument");
 	g_object_set (G_OBJECT (action),  "sensitive", FALSE, NULL);
 	break;
-	
+
     }
 }
 
@@ -1007,8 +1043,8 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
     if (type == YELP_RRN_TYPE_ERROR) {
 	gchar *message = g_strdup_printf (_("The requested URI \"%s\" is invalid"), trace_uri);
 	window_error (window, _("Unable to load page"), message, FALSE);
-	g_free (message);
-	return;
+
+	goto Exit;
     }
 
     if (priv->uri && g_str_equal (real_uri, priv->uri)) {
@@ -1022,6 +1058,10 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
 	    doc = yelp_toc_get ();
 	    priv->base_uri = g_strdup ("file:///fakefile");
 	    break;
+        case YELP_RRN_TYPE_MAL:
+            priv->base_uri = g_filename_to_uri (real_uri, NULL, NULL);
+            doc = yelp_mallard_new (real_uri);
+            break;
 	case YELP_RRN_TYPE_MAN:
 	    priv->base_uri = g_filename_to_uri (real_uri, NULL, NULL);
 	    doc = yelp_man_new (real_uri);
@@ -1046,15 +1086,17 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
 	    break;
 	case YELP_RRN_TYPE_HTML:
 	case YELP_RRN_TYPE_XHTML:
-	    priv->base_uri = g_strdup ("file:///fakefile");
-	    window_do_load_html (window, real_uri, frag_id, type, TRUE);
+	case YELP_RRN_TYPE_TEXT:
+ 	    {
+		gchar *call_uri;
+		priv->base_uri = g_strdup ("file:///fakefile");
+		call_uri = g_filename_to_uri (real_uri, NULL, NULL);
+		window_do_load_html (window, call_uri, frag_id, type, TRUE);
+		g_free (call_uri);
+ 	    }
 	    break;
 	case YELP_RRN_TYPE_EXTERNAL:
 	    {
-		gchar *str_stdout = NULL;
-		gchar *str_stderr = NULL;
-		gchar *cmd = NULL;
-		gint status = 0;
 		GError *error = NULL;
 
 		priv->base_uri = old_base_uri;
@@ -1066,24 +1108,25 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
 		    window_error (window, _("Unable to load page"), message, FALSE);
 		    g_free (message);
 
-		    return;
+		    goto Exit;
 		}
 
-		cmd = g_strdup_printf ("gnome-open %s", uri);
-		if (!g_spawn_command_line_sync (cmd, &str_stdout, &str_stderr, &status, &error)) {
-		    g_free (error);
-		    error = NULL;
-		    g_free (cmd);
-		    cmd = g_strdup_printf ("xdg-open %s", uri);
-		    if (!g_spawn_command_line_sync (cmd, &str_stdout, &str_stderr, &status, &error)) {
-			window_error(window, _("Error executing \"gnome-open\""), error->message, FALSE);
-			return;
-		    }
-		}
-		if (status) {
+		if (!gtk_show_uri (NULL, trace_uri, gtk_get_current_event_time (), &error)) {
 		    gchar *message = g_strdup_printf (_("The requested URI \"%s\" is invalid"), trace_uri);
 		    window_error (window, _("Unable to load page"), message, FALSE);
-		    return;
+		    /* todo: Freeing error here causes an immediate 
+		     * seg fault.  Why?  Who knows. For now, we'll
+		     * leak it
+		     */
+		    /* g_free (error); */
+		    error = NULL;
+		    if (!priv->current_document) {
+			/* recurse to load the index if we're not
+			 * already somewhere
+			 */
+			yelp_window_load (window, "x-yelp-toc:");
+		    }
+		    goto Exit;
 		}
 	    }
 	    break;
@@ -1097,22 +1140,39 @@ yelp_window_load (YelpWindow *window, const gchar *uri)
     }
 
     if (doc) {
+        gchar *slash;
 	gboolean need_hist = FALSE;
 	if (!frag_id)
 	  frag_id = g_strdup ("x-yelp-index");
 
 	if (priv->current_document || (priv->current_type == YELP_RRN_TYPE_HTML ||
-				       priv->current_type == YELP_RRN_TYPE_XHTML))
+				       priv->current_type == YELP_RRN_TYPE_XHTML ||
+				       priv->current_type == YELP_RRN_TYPE_TEXT))
 	    need_hist = TRUE;
 	window_setup_window (window, type, real_uri, frag_id,
 			     (gchar *) uri, current_base, need_hist);
 
-	priv->current_request = yelp_document_get_page (doc, 
-							frag_id, 
-							(YelpDocumentFunc) page_request_cb, 
+        /* FIXME: Super hacky.  We want the part before the slash for mallard IDs,
+           because right now we're outputting "#page_id/section_id".  We ought to
+           be scrolling to the section as well.  I happen to know that get_page is
+           just going to strdup the string, and we free frag_id itself immediately
+           after this, so this is safe.  It just sucks.
+         */
+        slash = strchr (frag_id, '/');
+        if (slash)
+            *slash = '\0';
+	priv->current_request = yelp_document_get_page (doc,
+							frag_id,
+							(YelpDocumentFunc) page_request_cb,
 							(void *) window);
 	priv->current_document = doc;
     }
+
+ Exit:
+    g_free (frag_id);
+    g_free(real_uri);
+    g_free(trace_uri);
+    g_free(current_base);
 }
 
 GtkUIManager *
@@ -1180,17 +1240,17 @@ search_activated (GtkAction *action,
      * the relevant page
      * Trigger it using "man:<foo>", "man <foo>" or "info:<foo>"
      */
-    if (g_str_has_prefix (search_terms, "man:") || 
+    if (g_str_has_prefix (search_terms, "man:") ||
 	g_str_has_prefix (search_terms, "info:")) {
 	uri = g_strdup (search_terms);
-    } else if (g_str_has_prefix (search_terms, "man ") && 
+    } else if (g_str_has_prefix (search_terms, "man ") &&
 	       !strstr (&(search_terms[4])," ")) {
 	uri = g_strdup (search_terms);
 	uri[3]=':';
     } else if (g_str_has_prefix (search_terms, "info ")) {
 	gint count = 0;
 	gchar *spaces;
-	
+
 	spaces = strchr (search_terms, ' ');
 	while (spaces) {
 	    count++;
@@ -1248,7 +1308,7 @@ window_populate (YelpWindow *window)
 
     f_proxy = GTK_WIDGET (gtk_menu_tool_button_new (NULL, NULL));
 
-    gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (b_proxy), 
+    gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (b_proxy),
 				   priv->back_menu);
 
     action = gtk_action_group_get_action(priv->action_group, "GoBack");
@@ -1258,16 +1318,16 @@ window_populate (YelpWindow *window)
     action = gtk_action_group_get_action (priv->action_group, "GoForward");
     gtk_action_connect_proxy (action, f_proxy);
 
-    gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (f_proxy), 
+    gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (f_proxy),
 				   priv->forward_menu);
-    
+
     priv->search_action =  (GtkWidget * )gtk_entry_action_new ("Search",
 				    _("_Search:"),
 				    _("Search for other documentation"),
 				    NULL);
     g_signal_connect (G_OBJECT (priv->search_action), "activate",
 		      G_CALLBACK (search_activated), window);
-    gtk_action_group_add_action (priv->action_group, 
+    gtk_action_group_add_action (priv->action_group,
 				 (GtkAction *) priv->search_action);
 
     priv->ui_manager = gtk_ui_manager_new ();
@@ -1317,9 +1377,9 @@ window_populate (YelpWindow *window)
     priv->maillink = gtk_ui_manager_get_widget(priv->ui_manager, "ui/mail_popup");
     priv->merge_id = gtk_ui_manager_new_merge_id (priv->ui_manager);
 
-    priv->find_next_menu = gtk_action_group_get_action (priv->action_group, 
+    priv->find_next_menu = gtk_action_group_get_action (priv->action_group,
 							"FindNext");
-    priv->find_prev_menu = gtk_action_group_get_action (priv->action_group, 
+    priv->find_prev_menu = gtk_action_group_get_action (priv->action_group,
 						      "FindPrev");
 
     priv->pane = gtk_hpaned_new ();
@@ -1420,7 +1480,7 @@ static gboolean
 window_key_event_cb (GtkWidget *widget, GdkEventKey *event,
 		     YelpWindow *window)
 {
-    if ((window->priv->search_action && 
+    if ((window->priv->search_action &&
 	 gtk_entry_action_has_focus ((GtkEntryAction *) window->priv->search_action)) ||
 	GTK_WIDGET_HAS_FOCUS (window->priv->find_entry))
 	return FALSE;
@@ -1434,7 +1494,7 @@ window_key_event_cb (GtkWidget *widget, GdkEventKey *event,
 }
 
 static gboolean
-window_find_hide_cb (GtkWidget *widget, GdkEventFocus *event, 
+window_find_hide_cb (GtkWidget *widget, GdkEventFocus *event,
 		     YelpWindow *window)
 {
     YelpWindowPriv *priv;
@@ -1457,7 +1517,7 @@ window_populate_find (YelpWindow *window,
     GtkToolItem *item;
     GtkWidget *arrow;
     YelpWindowPriv *priv = window->priv;
-    
+
     g_return_if_fail (GTK_IS_TOOLBAR (find_bar));
 
     box = gtk_hbox_new (FALSE, 0);
@@ -1515,7 +1575,7 @@ window_populate_find (YelpWindow *window,
     gtk_container_add (GTK_CONTAINER (priv->find_not_found), label);
     gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
     gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
-    gtk_toolbar_insert (GTK_TOOLBAR (find_bar), priv->find_not_found, -1);    
+    gtk_toolbar_insert (GTK_TOOLBAR (find_bar), priv->find_not_found, -1);
 
     item = gtk_separator_tool_item_new ();
     gtk_tool_item_set_expand (item, TRUE);
@@ -1541,7 +1601,7 @@ window_set_sections (YelpWindow   *window,
     priv = window->priv;
 
 	gtk_tree_view_set_model (GTK_TREE_VIEW (priv->side_sects), sections);
-	
+
 	if (sections) {
 	    gtk_widget_show_all (priv->side_sw);
 	    window_set_section_cursor (window, sections);
@@ -1563,7 +1623,7 @@ window_set_section_cursor (YelpWindow * window, GtkTreeModel *model)
 				     tree_selection_changed_cb,
 				     window);
     gtk_tree_selection_unselect_all (selection);
-    
+
     valid = gtk_tree_model_get_iter_first (model, &iter);
     while (valid) {
 	gtk_tree_model_get (model, &iter,
@@ -1580,14 +1640,14 @@ window_set_section_cursor (YelpWindow * window, GtkTreeModel *model)
 	    }
 	    path = gtk_tree_model_get_path (model, &iter);
 	    gtk_tree_selection_select_path (selection, path);
-	    
+
 	    gtk_tree_path_free (path);
 	    g_free (id);
 	    break;
 	}
-	
+
 	g_free (id);
-	
+
 	valid = tree_model_iter_following (model, &iter);
     }
     g_signal_handlers_unblock_by_func (selection,
@@ -1609,7 +1669,6 @@ window_do_load_html (YelpWindow    *window,
     gchar             buffer[BUFFER_SIZE];
     GtkAction        *action;
     gchar *real_uri = NULL;
-    gchar *base_uri = NULL;
 
     gboolean  handled = TRUE;
 
@@ -1628,10 +1687,13 @@ window_do_load_html (YelpWindow    *window,
     action = gtk_action_group_get_action (priv->action_group, "GoContents");
     if (action)
 	g_object_set (G_OBJECT (action), "sensitive", FALSE, NULL);
-    
+
     window_setup_window (window, type, uri, frag_id, uri, priv->base_uri, need_history);
 
-    file   = g_file_new_for_uri (uri);
+    if (uri[0] == '/')
+	file = g_file_new_for_path (uri);
+    else
+	file = g_file_new_for_uri (uri);
     stream = g_file_read (file, NULL, NULL);
 
     if (stream == NULL) {
@@ -1646,12 +1708,10 @@ window_do_load_html (YelpWindow    *window,
 	goto done;
     }
 
-    base_uri = g_filename_to_uri (uri, NULL, NULL);
     if (frag_id) {
-	real_uri = g_strconcat (base_uri, "#", frag_id, NULL);
-	g_free (base_uri);
+	real_uri = g_strconcat (uri, "#", frag_id, NULL);
     } else {
-	real_uri = base_uri;
+	real_uri = g_strdup (uri);
     }
     yelp_html_set_base_uri (priv->html_view, real_uri);
 
@@ -1662,13 +1722,20 @@ window_do_load_html (YelpWindow    *window,
     case YELP_RRN_TYPE_XHTML:
 	yelp_html_open_stream (priv->html_view, "application/xhtml+xml");
 	break;
+    case YELP_RRN_TYPE_TEXT:
+	yelp_html_open_stream (priv->html_view, "text/plain");
+	break;
     default:
 	g_assert_not_reached ();
     }
 
     while ((g_input_stream_read_all
-	    ((GInputStream *)stream, buffer, BUFFER_SIZE, &n, NULL, NULL))) {
+	    ((GInputStream *)stream, buffer, BUFFER_SIZE, &n, NULL, NULL)) && n) {
 	gchar *tmp;
+
+	if (n == 0)
+		break;
+
 	tmp = g_utf8_strup (buffer, n);
 	if (strstr (tmp, "<FRAMESET")) {
 	    yelp_html_frames (priv->html_view, TRUE);
@@ -1730,10 +1797,33 @@ window_configure_cb (GtkWidget         *widget,
 		     gpointer           data)
 {
     gint width, height;
+    GKeyFile *keyfile;
+    GError *config_error = NULL;
+    gchar *sdata, *config_path;
+    gsize config_size;
+
     gtk_window_get_size (GTK_WINDOW (widget), &width, &height);
-    gnome_config_set_int (YELP_CONFIG_WIDTH, width);
-    gnome_config_set_int (YELP_CONFIG_HEIGHT, height);
-    gnome_config_sync ();
+
+    keyfile = g_key_file_new();
+
+    config_path = g_strconcat (g_get_home_dir(), YELP_CONFIG_PATH, NULL);
+
+    g_key_file_set_integer (keyfile, YELP_CONFIG_GEOMETRY_GROUP,
+			    YELP_CONFIG_WIDTH, width);
+    g_key_file_set_integer (keyfile, YELP_CONFIG_GEOMETRY_GROUP,
+			    YELP_CONFIG_HEIGHT, height);
+
+    sdata = g_key_file_to_data (keyfile, &config_size, NULL);
+
+    if ( !g_file_set_contents (config_path, sdata,
+			       config_size, &config_error) ) {
+	g_warning ("Failed to save config file: %s\n", config_error->message);
+	g_error_free (config_error);
+    }
+
+    g_free (sdata);
+    g_free (config_path);
+    g_key_file_free (keyfile);
 
     return FALSE;
 }
@@ -1766,6 +1856,7 @@ html_frame_selected_cb (YelpHtml *html, gchar *uri, gboolean handled,
     switch (window->priv->current_type) {
     case YELP_RRN_TYPE_XHTML:
     case YELP_RRN_TYPE_HTML:
+    case YELP_RRN_TYPE_TEXT:
 	handle = TRUE;
 	break;
     default:
@@ -1953,7 +2044,7 @@ window_print_signal (YelpDocument       *document,
 	break;
     case YELP_DOCUMENT_SIGNAL_ERROR:
 	error = (YelpError *) func_data;
-	window_error (print->window, (gchar *) yelp_error_get_title (error), 
+	window_error (print->window, (gchar *) yelp_error_get_title (error),
 		      (gchar *) yelp_error_get_message (error), FALSE);
 	yelp_error_free (error);
 	break;
@@ -1973,7 +2064,7 @@ window_print_document_cb (GtkAction *action, YelpWindow *window)
     GtkWidget *vbox = gtk_vbox_new (FALSE, FALSE);
     PrintStruct *print;
     YelpDocument *doc = NULL;
-    
+
     priv = window->priv;
 
     switch (priv->current_type) {
@@ -1988,7 +2079,7 @@ window_print_document_cb (GtkAction *action, YelpWindow *window)
 
     gtk_win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     html = yelp_html_new ();
-    
+
     gtk_container_add (GTK_CONTAINER (gtk_win), GTK_WIDGET (vbox));
     gtk_box_pack_end (GTK_BOX (vbox), GTK_WIDGET (html), TRUE, TRUE, 0);
     gtk_widget_show (gtk_win);
@@ -1997,7 +2088,7 @@ window_print_document_cb (GtkAction *action, YelpWindow *window)
     gtk_widget_hide (gtk_win);
 
     print = g_new0 (PrintStruct, 1);
-    
+
     print->window = window;
     print->gtk_win = (GtkWindow *) gtk_win;
     print->vbox = (GtkVBox *) vbox;
@@ -2005,7 +2096,7 @@ window_print_document_cb (GtkAction *action, YelpWindow *window)
 
     yelp_document_get_page (doc,
 			    "x-yelp-index",
-			    (YelpDocumentFunc) window_print_signal, 
+			    (YelpDocumentFunc) window_print_signal,
 			    (void *) print);
 }
 
@@ -2017,12 +2108,12 @@ window_print_page_cb (GtkAction *action, YelpWindow *window)
     YelpHtml *html;
     GtkWidget *vbox = gtk_vbox_new (FALSE, FALSE);
     PrintStruct *print;
-    
+
     priv = window->priv;
 
     gtk_win = gtk_window_new (GTK_WINDOW_TOPLEVEL);
     html = yelp_html_new ();
-    
+
     gtk_container_add (GTK_CONTAINER (gtk_win), GTK_WIDGET (vbox));
     gtk_box_pack_end (GTK_BOX (vbox), GTK_WIDGET (html), TRUE, TRUE, 0);
     gtk_widget_show (gtk_win);
@@ -2031,7 +2122,7 @@ window_print_page_cb (GtkAction *action, YelpWindow *window)
     gtk_widget_hide (gtk_win);
 
     print = g_new0 (PrintStruct, 1);
-    
+
     print->window = window;
     print->gtk_win = (GtkWindow *) gtk_win;
     print->vbox = (GtkVBox *) vbox;
@@ -2040,9 +2131,9 @@ window_print_page_cb (GtkAction *action, YelpWindow *window)
 
     if (priv->current_document) {
 	/* Need to go through the paging system */
-	yelp_document_get_page (priv->current_document, 
-				priv->current_frag, 
-				(YelpDocumentFunc) window_print_signal, 
+	yelp_document_get_page (priv->current_document,
+				priv->current_frag,
+				(YelpDocumentFunc) window_print_signal,
 				(void *) print);
 
     } else {
@@ -2051,11 +2142,11 @@ window_print_page_cb (GtkAction *action, YelpWindow *window)
 	GFile            *file;
 	GFileInputStream *stream;
 	gsize             n;
-	gchar             buffer[BUFFER_SIZE];	
-	
+	gchar             buffer[BUFFER_SIZE];
+
     file   = g_file_new_for_uri (priv->uri);
     stream = g_file_read (file, NULL, NULL);
-	
+
 	if (stream == NULL) {
 	    /*GError *error = NULL;
 	    g_set_error (&error, YELP_ERROR, YELP_ERROR_IO,
@@ -2083,10 +2174,13 @@ window_print_page_cb (GtkAction *action, YelpWindow *window)
 	case YELP_RRN_TYPE_XHTML:
 	    yelp_html_open_stream (html, "application/xhtml+xml");
 	    break;
+	case YELP_RRN_TYPE_TEXT:
+	    yelp_html_open_stream (html, "text/plain");
+	    break;
 	default:
 	    g_assert_not_reached ();
 	}
-	
+
 	while ((g_input_stream_read_all
 	    ((GInputStream *)stream, buffer, BUFFER_SIZE, &n, NULL, NULL))) {
 	    yelp_html_write (html, buffer, n);
@@ -2096,11 +2190,11 @@ window_print_page_cb (GtkAction *action, YelpWindow *window)
         g_object_unref (file);
     if (stream)
         g_object_unref (stream);
-	
+
 	yelp_html_close (html);
-	
+
 	yelp_print_run (window, html, gtk_win, vbox);
-	
+
     }
 }
 
@@ -2124,7 +2218,8 @@ static void
 window_open_location_cb (GtkAction *action, YelpWindow *window)
 {
     YelpWindowPriv *priv;
-    GladeXML       *glade;
+    GtkBuilder     *builder;
+    GError         *error = NULL;
     GtkWidget      *dialog;
     GtkWidget      *entry;
     gchar          *uri = NULL;
@@ -2133,17 +2228,17 @@ window_open_location_cb (GtkAction *action, YelpWindow *window)
 
     priv = window->priv;
 
-    glade = glade_xml_new (DATADIR "/yelp/ui/yelp.glade",
-			   "location_dialog",
-			   NULL);
-    if (!glade) {
-	g_warning ("Could not find necessary glade file "
-		   DATADIR "/yelp/ui/yelp.glade");
-	return;
+    builder = gtk_builder_new ();
+    if (!gtk_builder_add_from_file (builder,
+                                    DATADIR "/yelp/ui/yelp-open-location.ui",
+                                    &error)) {
+        g_warning ("Could not load builder file: %s", error->message);
+        g_error_free(error);
+        return;
     }
 
-    dialog = glade_xml_get_widget (glade, "location_dialog");
-    entry  = glade_xml_get_widget (glade, "location_entry");
+    dialog = GTK_WIDGET (gtk_builder_get_object (builder, "location_dialog"));
+    entry  = GTK_WIDGET (gtk_builder_get_object (builder, "location_entry"));
 
     priv->location_dialog = dialog;
     priv->location_entry  = entry;
@@ -2161,7 +2256,6 @@ window_open_location_cb (GtkAction *action, YelpWindow *window)
 		      G_CALLBACK (location_response_cb),
 		      window);
 
-    g_object_unref (glade);
     gtk_window_present (GTK_WINDOW (dialog));
 }
 
@@ -2199,7 +2293,7 @@ window_select_all_cb (GtkAction *action, YelpWindow *window)
     if (GTK_IS_EDITABLE (widget)) {
 	gtk_editable_select_region (GTK_EDITABLE (widget), 0, -1);
     } else {
-	yelp_html_select_all (window->priv->html_view);	
+	yelp_html_select_all (window->priv->html_view);
     }
 }
 
@@ -2255,18 +2349,19 @@ history_load_entry (YelpWindow *window, YelpHistoryEntry *entry)
 {
     g_return_if_fail (YELP_IS_WINDOW (window));
 
-    if (entry->type == YELP_RRN_TYPE_HTML || entry->type == YELP_RRN_TYPE_XHTML) {
+    if (entry->type == YELP_RRN_TYPE_HTML || entry->type == YELP_RRN_TYPE_XHTML || entry->type == YELP_RRN_TYPE_TEXT) {
 	window_do_load_html (window, entry->uri, entry->frag_id, entry->type, FALSE);
     } else {
 	g_assert (entry->doc != NULL);
 	window_setup_window (window, entry->type, entry->uri, entry->frag_id, entry->req_uri,
 			     window->priv->base_uri, FALSE);
-	g_free (window->priv->base_uri);
+	if (window->priv->base_uri)
+	    g_free (window->priv->base_uri);
 	window->priv->base_uri = g_strdup (entry->base_uri);
 	window->priv->current_document = entry->doc;
-	window->priv->current_request = yelp_document_get_page (entry->doc, 
-							entry->frag_id, 
-							(YelpDocumentFunc) page_request_cb, 
+	window->priv->current_request = yelp_document_get_page (entry->doc,
+							entry->frag_id,
+							(YelpDocumentFunc) page_request_cb,
 							(void *) window);
     }
 
@@ -2404,20 +2499,18 @@ window_add_bookmark_cb (GtkAction *action, YelpWindow *window)
 
 }
 
-static void window_copy_link_cb (GtkAction *action, YelpWindow *window) 
+static void window_copy_link_cb (GtkAction *action, YelpWindow *window)
 {
-    gtk_clipboard_set_text (gtk_clipboard_get (gdk_atom_intern ("CLIPBOARD", 
+    gtk_clipboard_set_text (gtk_clipboard_get (gdk_atom_intern ("CLIPBOARD",
 								TRUE)),
 			    window->priv->uri,
 			    -1);
-    g_free (window->priv->uri);
 }
 
 static void
 window_open_link_cb (GtkAction *action, YelpWindow *window)
 {
     yelp_window_load (window, window->priv->uri);
-    g_free (window->priv->uri);
 }
 
 static void
@@ -2425,7 +2518,6 @@ window_open_link_new_cb (GtkAction *action, YelpWindow *window)
 {
     g_signal_emit (window, signals[NEW_WINDOW_REQUESTED], 0,
 		   window->priv->uri);
-    g_free (window->priv->uri);
 }
 
 /* TODO: This doesn't work... */
@@ -2437,10 +2529,10 @@ window_copy_mail_cb (GtkAction *action, YelpWindow *window)
      * remove the first 7 chars as they should be mailto:
      */
     gchar **split_string = g_strsplit (window->priv->uri, "?", 2);
-    
+
     gchar *mail_address = &split_string[0][7];
-    
-    gtk_clipboard_set_text (gtk_clipboard_get (gdk_atom_intern ("CLIPBOARD", 
+
+    gtk_clipboard_set_text (gtk_clipboard_get (gdk_atom_intern ("CLIPBOARD",
 								TRUE)),
 			    mail_address,
 			    -1);
@@ -2455,7 +2547,7 @@ window_help_contents_cb (GtkAction *action, YelpWindow *window)
      * we can make this function easy by just creating a new window
      * and avoiding calling gnome_help_display_desktop
      */
-    g_signal_emit (window, signals[NEW_WINDOW_REQUESTED], 0, 
+    g_signal_emit (window, signals[NEW_WINDOW_REQUESTED], 0,
 		   "ghelp:user-guide#yelp");
 }
 
@@ -2467,7 +2559,7 @@ window_about_cb (GtkAction *action, YelpWindow *window)
 	"Copyright © 2003-2005 Shaun McCance\n"
 	"Copyright © 2005-2006 Don Scorgie\n"
 	"Copyright © 2005-2006 Brent Smith";
-    const gchar *authors[] = { 
+    const gchar *authors[] = {
 	"Mikael Hallendal <micke@imendio.com>",
 	"Alexander Larsson <alexl@redhat.com>",
 	"Shaun McCance <shaunm@gnome.org>",
@@ -2502,9 +2594,9 @@ location_response_cb (GtkDialog *dialog, gint id, YelpWindow *window)
     debug_print (DB_ARG, "  id = %i\n", id);
 
     if (id == GTK_RESPONSE_OK) {
-	const gchar *uri = 
+	const gchar *uri =
 	    gtk_entry_get_text (GTK_ENTRY (window->priv->location_entry));
-    
+
 	yelp_window_load (window, uri);
     }
 
@@ -2526,7 +2618,7 @@ window_find_save_settings (YelpWindow *window)
     tmp = gtk_entry_get_text (GTK_ENTRY (priv->find_entry));
 
     g_free (priv->find_string);
-		
+
     priv->find_string = g_utf8_casefold (tmp, -1);
 }
 
@@ -2614,7 +2706,7 @@ window_find_entry_changed_cb (GtkEditable *editable,
     gboolean        found;
 
     g_return_if_fail (YELP_IS_WINDOW(data));
-	
+
     window = YELP_WINDOW (data);
     priv = window->priv;
 
@@ -2634,7 +2726,7 @@ window_find_entry_changed_cb (GtkEditable *editable,
 	    gtk_widget_show_all (GTK_WIDGET (priv->find_not_found));
 	    window_find_buttons_set_sensitive (window, TRUE, TRUE);
 	}
- 
+
     g_free (text);
 }
 
@@ -2692,9 +2784,9 @@ window_find_buttons_set_sensitive (YelpWindow  *window,
 
     gtk_widget_set_sensitive (GTK_WIDGET (priv->find_next), next);
     gtk_widget_set_sensitive (GTK_WIDGET (priv->find_prev), prev);
-    g_object_set (G_OBJECT (priv->find_next_menu), "sensitive", next, 
+    g_object_set (G_OBJECT (priv->find_next_menu), "sensitive", next,
 		  NULL);
-    g_object_set (G_OBJECT (priv->find_prev_menu), "sensitive", prev, 
+    g_object_set (G_OBJECT (priv->find_prev_menu), "sensitive", prev,
 		  NULL);
 }
 
@@ -2741,18 +2833,18 @@ window_write_html (YelpLoadData *data)
     gsize read;
     YelpHtml *html = data->window->priv->html_view;
     gchar contents[BUFFER_SIZE];
-    
+
     /* Use a silly fake URI to stop gecko doing silly things */
     if (data->window->priv->current_frag)
-	uri = g_strdup_printf ("%s#%s", data->window->priv->base_uri, 
+	uri = g_strdup_printf ("%s#%s", data->window->priv->base_uri,
 			       data->window->priv->current_frag);
     else
 	uri = g_strdup (data->window->priv->base_uri);
-    
+
     yelp_html_set_base_uri (html, uri);
     g_free (uri);
     yelp_html_open_stream (html, "application/xhtml+xml");
-    
+
     do {
 	yelp_page_read (data->page, contents, BUFFER_SIZE, &read, NULL);
 	yelp_html_write (html, contents, read);
@@ -2766,11 +2858,11 @@ window_write_print_html (YelpHtml *html, YelpPage * page)
 {
     gsize read;
     gchar contents[BUFFER_SIZE];
-    
+
     /* Use a silly fake URI to stop gecko doing silly things */
     yelp_html_set_base_uri (html, "file:///foobar");
     yelp_html_open_stream (html, "application/xhtml+xml");
-    
+
     do {
 	yelp_page_read (page, contents, BUFFER_SIZE, &read, NULL);
 	yelp_html_write (html, contents, read);
